/*
 *  Copyright 2010 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

Ext.namespace('org.ametys.repository');


/**
 * Tab panel for JCR repository
 * @param standalone true if the application is use in standalone mode
 */
org.ametys.repository.JCRViewTab = function (standalone)
{
	this._standalone = standalone;
	
	// The JCR tree
	this._jcrTree = new org.ametys.repository.tree.JCRTreePanel ({
		region: 'center',
		split: true,
		id: 'jcr-tree'
	});
	this._jcrTree.getSelectionModel().addListener('selectionchange', org.ametys.repository.JCRViewTab.onSelectNode, this);
	
	// The tree for search result
	this._searchTree = new org.ametys.repository.tree.SearchTreePanel ({
		region: 'south',
		split: true,
		loaderUrl: context.workspaceContext + '/repository/jcr-query',
		iconNodeUrl: context.workspaceContext + '/resources/img/jcr_node_16.png'
	});
	this._searchTree.getSelectionModel().addListener('selectionchange',  org.ametys.repository.JCRViewTab._selectJCRNode, this);
	
	// Trees = WorkspaceChoice + JCR Tree + Search Tree
	this._treesPanel = new Ext.Panel ({
		layout: 'border',
		region: 'west',
		width: 350,
		
		split: true,
		autoScroll: false,
		border: false,
	
		items: [ this._jcrTree, 
		         this._searchTree
	         ]
	});

	// Toolbar Panel
	this._toolbarPanel = new Ext.Panel ({
		region: 'north',
		split: false,
		height: 45,
		
		layout: 'card',
		activeItem: 0,
		
		layoutOnCardChange: true
	});
	
	// Toolbar
	this._toolbar = org.ametys.repository.JCRViewTab._getToolbar (this);
	
	this._toolbarPanel.add(this._toolbar);
	this._toolbarPanel.doLayout();
	
	// Path of the selected node
	this._pathPanel = new Ext.Panel({
		region: 'north',
		
		border: false,
		cls: 'path-panel',
		html: "",
		hidden: true
	});
	
	// Properties
	this._propertiesFormPanel = new org.ametys.repository.PropertiesFormPanel ({
        border: false,
        
        jcrView: this,
        
		items: []
	});
	
	// Referers grid
	this._referersPanel = org.ametys.repository.JCRViewTab._getReferersPanel (this);

	// Info = (Path) + (Instructions+Properties) + (Referers)
	this._infoPanel = new Ext.Panel ({
		region: 'center',
		split:true,
		autoScroll: true,
		layout: 'fit',
		items: new Ext.Panel ({
			autoScroll: true,
			border: false,
			items : [ 
		        this._pathPanel,
		        this._propertiesFormPanel ,
		        this._referersPanel
	         ]
		})
	});
	
	/**
	 * Returns the global planel
	 */
	return new Ext.Panel ({
		title: "<i18n:text i18n:key="PHYSICAL_VIEW_TAB_TITLE"/>",
		id: 'phys-tab',
		layout: 'border',
		region: 'center',
	
		items: [ 
		        this._toolbarPanel, //north
		        this._treesPanel, //west
		        this._infoPanel //center
	         ]
	});
	
}

/** Properties. */
org.ametys.repository.JCRViewTab.hasPendingChanges = false;
org.ametys.repository.JCRViewTab.nodesPendingChanges = [];
org.ametys.repository.JCRViewTab.hasModifiedAttributes = false;

org.ametys.repository.JCRViewTab._getReferersPanel = function (view)
{
	
	return  new Ext.grid.GridPanel({
		title: "<i18n:text i18n:key="REFERERS_PANEL_TITLE"/>",
		
		cls:'referencers-panel',
		region: 'south',
		
		collapsible: true,
		collapsed: true,
		titleCollapse: true,
		animCollapse: true,
		hideCollapseTool: true,
		floatable: false,
		
		hidden: false,
        forceLayout: true,
		autoScroll: false,
		border: false,
		
	    store: new Ext.data.ArrayStore({
		    autoDestroy: true,
		    idIndex: 0,  
		    fields: [
		       {name: 'path'},
		       {name: 'property'}
		    ]
		}),
	    
	    columns: [
			{	
				header: "<i18n:text i18n:key="REFERERS_PANEL_PATH_COLUMN_HEADER"/>", 
				id:"path", 
				width: 320, 
				sortable: true, 
				dataIndex: 'path',
		        renderer: function (value, metaData, record, rowIndex, colIndex, store){ return '&lt;a href="#" onClick="org.ametys.repository.JCRViewTab.selectNodeByPath (\'' + view._jcrTree.getId() + '\', \'' + value + '\'); return(false);"&gt;' + value + '&lt;/a&gt;'}
			},
	        {header: "<i18n:text i18n:key="REFERERS_PANEL_PROPERTY_COLUMN_HEADER"/>", width: 150, sortable: true, dataIndex: 'property'}
	    ]
	});
}

org.ametys.repository.JCRViewTab.selectNodeByPath = function (treeId, value)
{
	Ext.getCmp(treeId).selectNodeByPath (value);
}

org.ametys.repository.JCRViewTab._getToolbar = function (view)
{
    var sortMenu = new Ext.menu.Menu({
        id: 'sort-menu',
        items: [
            {
                id: 'sort-menu-item-document',
                text: "<i18n:text i18n:key='NODES_DOCUMENT_ORDER_BUTTON_TITLE'/>",
                description: "<i18n:text i18n:key='NODES_DOCUMENT_ORDER_BUTTON_BODY'/>",
                icon: context.workspaceContext + '/resources/img/sort-document-32.png',
                scope: view,
                handler: org.ametys.repository.JCRViewTab._onSortChange,
                order: 'document'
            }, {
                id: 'sort-menu-item-alpha',
                text: "<i18n:text i18n:key='NODES_ALPHA_ORDER_BUTTON_TITLE'/>",
                description: "<i18n:text i18n:key='NODES_ALPHA_ORDER_BUTTON_BODY'/>",
                icon: context.workspaceContext + '/resources/img/sort-alpha-32.png',
                scope: view,
                handler: org.ametys.repository.JCRViewTab._onSortChange,
                order: 'alpha'
            }, {
                id: 'sort-menu-item-reverseAlpha',
                text: "<i18n:text i18n:key='NODES_REVERSE_ALPHA_ORDER_BUTTON_TITLE'/>",
                description: "<i18n:text i18n:key='NODES_REVERSE_ALPHA_ORDER_BUTTON_BODY'/>",
                icon: context.workspaceContext + '/resources/img/sort-alpha-reverse-32.png',
                scope: view,
                handler: org.ametys.repository.JCRViewTab._onSortChange,
                order: 'reverseAlpha'
            }
        ]
    });
    
	var toolbar = new Ext.Toolbar ({
		region: 'north',
		items : [
		         // refresh node
		         {
                     id: 'btn-node-refresh',
					 icon: context.workspaceContext + '/resources/img/reload_32.png',
					 tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="REFRESH_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="REFRESH_BUTTON_TOOLTIP_BODY"/>", context.workspaceContext + '/resources/img/reload_32.png'),
					 scale: 'large',
					 scope: view,
		        	 handler: org.ametys.repository.JCRViewTab._refreshNode
		         },	
		         '-',
		         // add a node
		         {
                     id: 'btn-node-add',
					 icon: context.workspaceContext + '/resources/img/add_node_32.png',
					 tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="ADD_NODE_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="ADD_NODE_BUTTON_TOOLTIP_BODY"/>", context.workspaceContext + '/resources/img/add_node_32.png'),
					 scale: 'large',
					 scope: view,
		        	 handler: org.ametys.repository.JCRViewTab._addNode
		         },
		         // delete a node
		         {
                     id: 'btn-node-remove',
					 icon: context.workspaceContext + '/resources/img/delete_node_32.png', 
					 tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="REMOVE_NODE_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="REMOVE_NODE_BUTTON_TOOLTIP_BODY"/>", context.workspaceContext + '/resources/img/delete_node_32.png'),
					 scale: 'large',
					 scope: view,
		        	 handler: org.ametys.repository.JCRViewTab._removeNode
		         },
		         // add property
		         {
                     id: 'btn-property-add',
					 icon: context.workspaceContext + '/resources/img/add_properties_32.png', 
					 tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="CREATE_PROP_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="CREATE_PROP_BUTTON_TOOLTIP_BODY"/>", context.workspaceContext + '/resources/img/add_properties_32.png'),
					 scale: 'large',
					 scope: view,
					 handler: org.ametys.repository.JCRViewTab._addProperty
		         },
                 '-',
		         // save session
		         {
		             id: 'btn-save-session',
		             icon: context.workspaceContext + '/resources/img/save_32.png', 
		             tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='SAVE_SESSION_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='SAVE_SESSION_BUTTON_TOOLTIP_BODY'/>", context.workspaceContext + '/resources/img/save_32.png'),
		             scale: 'large',
		             scope: view,
		             handler: org.ametys.repository.JCRViewTab._saveSession,
		             disabled: true
		         },
		         // rollback session
		         {
		             id: 'btn-rollback-session',
		             icon: context.workspaceContext + '/resources/img/rollback_32.png', 
		             tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='ROLLBACK_SESSION_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='ROLLBACK_SESSION_BUTTON_TOOLTIP_BODY'/>", context.workspaceContext + '/resources/img/rollback_32.png'),
		             scale: 'large',
		             scope: view,
		             handler: org.ametys.repository.JCRViewTab._rollbackSession,
		             disabled: true
		         },
		         '-',
		         // lock status
				 {
					 id: 'btn-status-lock',
					 icon: context.workspaceContext + '/resources/img/unlock_32.png',
					 tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="LOCKED_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="LOCKED_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/unlock_32.png'),
					 scale: 'large',
					 scope: view,
					 // TODO  REPOSITORY-75 handler: org.ametys.repository.JCRViewTab._unlockNode,
					 disabled: true
				 },	   
				 // checkout status
				 {
					 id: 'btn-status-checkout',
					 icon: context.workspaceContext + '/resources/img/checkout_32.png',
					 tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="CHECKED_OUT_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="CHECKED_OUT_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/checkout_32.png'),
					 scale: 'large',
					 scope: view,
					 handler: org.ametys.repository.JCRViewTab._checkOutNode
				 },
                 '-',
                 // Sort menu item
                 {
				    id: 'sort-item',
			        icon: sortMenu.items.itemAt(0).icon,
			        tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent(sortMenu.items.itemAt(0).text, sortMenu.items.itemAt(0).description, sortMenu.items.itemAt(0).icon),
			        scale: 'large',
			        scope: view,
			        menu: sortMenu
                 },
		         '->',
				// execute XPath query
		        {
                    id: 'btn-search-xpath',
		        	icon: context.workspaceContext + '/resources/img/search_query_32.png',
		        	tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="XPATH_QUERY_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="XPATH_QUERY_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/search_query_32.png'),
		        	scale: 'large',
		        	scope: view,
		        	handler: org.ametys.repository.JCRViewTab._executeQuery
		         },
		         // search by UUID
		         {
                    id: 'btn-search-uuid',
		        	icon: context.workspaceContext + '/resources/img/search_uuid_32.png',
		        	tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="SOLVE_UUID_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="SOLVE_UUID_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/search_uuid_32.png'),
		        	scale: 'large',
		        	scope: view,
		        	handler: org.ametys.repository.JCRViewTab._solveUUID
		         }
				]
	});
	
	// Test the session state on rendering, to enable or disable the button.
	Ext.getCmp('btn-save-session').on('render', org.ametys.repository.JCRViewTab._onShowSaveSession, this);
//	Ext.getCmp('btn-rollback-session').on('render', org.ametys.repository.JCRViewTab._onShowSaveSession, this);
	
	if (view._standalone)
	{
		toolbar.add ({xtype: 'tbseparator'});
		// disconnect
		toolbar.add ({
			 id: 'btn-disconnect',
			 icon: context.workspaceContext + '/resources/img/disconnect_32.png', 
			 scale: 'large',
			 tooltip: org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="REPOSITORY_DISCONNECT_LABEL"/>", "<i18n:text i18n:key="REPOSITORY_DISCONNECT_DESC"/>", context.workspaceContext + '/resources/img/disconnect_32.png'),
			 scope: view,
			 handler: org.ametys.repository.JCRViewTab._disconnectRepository,
			 hidden: !view._standalone
		});
	}
	
	return toolbar;
}


/**
 * Update the properties panel
 * @param {Ext.tree.Node} The selected JCR node
 */
org.ametys.repository.JCRViewTab.onSelectNode = function (sm, node)
{
	if (node == null)
		return;
	
	org.ametys.repository.JCRViewTab._updateButtons(this, node);
	
	if (node.attributes.tagName != 'group')
	{
	    org.ametys.repository.JCRViewTab._updateNodeProperties (this, node);
	}
}

org.ametys.repository.JCRViewTab._updateButtons = function(view, node)
{
    var isGroup = node.attributes.tagName == 'group';
    
    if (isGroup)
    {
        var button = Ext.getCmp('btn-node-add');
        button.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='ADD_NODE_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='ADD_NODE_BUTTON_TOOLTIP_BODY_DISABLED'/>", context.workspaceContext + '/resources/img/add_node_32.png'));
        button.disable();
        
        button = Ext.getCmp('btn-node-remove');
        button.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='REMOVE_NODE_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='REMOVE_NODE_BUTTON_TOOLTIP_BODY_DISABLED'/>", context.workspaceContext + '/resources/img/delete_node_32.png'));
        button.disable();
        
        button = Ext.getCmp('btn-property-add');
        button.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='CREATE_PROP_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='CREATE_PROP_BUTTON_TOOLTIP_BODY_DISABLED'/>", context.workspaceContext + '/resources/img/add_properties_32.png'));
        button.disable();
    }
    else
    {
        var button = Ext.getCmp('btn-node-add');
        button.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='ADD_NODE_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='ADD_NODE_BUTTON_TOOLTIP_BODY'/>", context.workspaceContext + '/resources/img/add_node_32.png'));
        button.enable();
        
        button = Ext.getCmp('btn-node-remove');
        button.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='REMOVE_NODE_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='REMOVE_NODE_BUTTON_TOOLTIP_BODY'/>", context.workspaceContext + '/resources/img/delete_node_32.png'));
        button.enable();
        
        button = Ext.getCmp('btn-property-add');
        button.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent("<i18n:text i18n:key='CREATE_PROP_BUTTON_TOOLTIP_TITLE'/>", "<i18n:text i18n:key='CREATE_PROP_BUTTON_TOOLTIP_BODY'/>", context.workspaceContext + '/resources/img/add_properties_32.png'));
        button.enable();
    }
    
    org.ametys.repository.JCRViewTab._updateOrderMenu(this, node);
}

org.ametys.repository.JCRViewTab._updateOrderMenu = function(view, node)
{
    var item = Ext.getCmp('sort-item');
    
    if (node.attributes.hasOrderableChildNodes == 'true')
    {
        item.disable();
        item.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent(
            "<i18n:text i18n:key='SORT_DISABLED_HAS_ORDERABLE_CHILDREN_TITLE'/>",
            "<i18n:text i18n:key='SORT_DISABLED_HAS_ORDERABLE_CHILDREN_BODY'/>",
            context.workspaceContext + '/resources/img/sort-document-32.png')
        );
    }
    else if (node.attributes.tagName == 'group')
    {
        item.disable();
        item.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent(
            "<i18n:text i18n:key='SORT_DISABLED_GROUP_TITLE'/>",
            "<i18n:text i18n:key='SORT_DISABLED_GROUP_BODY'/>",
            context.workspaceContext + '/resources/img/sort-document-32.png')
        );
    }
    else
    {
        item.enable();
        var order = node.attributes.order || org.ametys.repository.defaultSort;
        var orderItem = Ext.getCmp('sort-menu-item-' + order);
        if (orderItem)
        {
            item.setIcon(orderItem.icon);
            item.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent(orderItem.text, orderItem.text, orderItem.icon));
        }
    }
}

org.ametys.repository.JCRViewTab._updateNodeProperties = function (view, node)
{
	var serverMessage = new org.ametys.servercomm.ServerMessage('_' + context.workspaceName, '/repository/node',
	        {
                workspace: view._jcrTree._workspaceName,
                path: node.attributes.path.substring(1)
            }, 
			org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS, 
			null, 
			this, 
			null);
	
	this._waitMsg = new org.ametys.msg.Mask(view._infoPanel.getEl(), "<i18n:text i18n:key="PROPERTIES_PANEL_WAITING_MSG"/>");
	
	var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
	
	if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key="ERROR_MSG_GET_PROPERTIES"/>", response, "org.ametys.repository.PropertiesFormPanel"))
	{
		if (this._waitMsg != null)
		{
			this._waitMsg.hide();
			delete this._waitMsg;
		}
		return;
	}
	
	// Update path
	view._pathPanel.show ();
	view._pathPanel.update("&lt;b&gt;&amp;gt; &lt;/b&gt;" + node.getPath('text') );
	
	// Update properties
	var properties = response.selectNodes("repository/node/property");
	view._propertiesFormPanel.updatePanel (view._jcrTree._workspaceName, node, properties);
	
	// Update referers
	var referers = response.selectNodes("repository/node/referer");
	org.ametys.repository.JCRViewTab.updateReferers (view._referersPanel, referers);
	
	// Update status buttons
	var isLocked = response.selectSingleNode("repository/node/isLocked")[org.ametys.servercomm.ServerComm.xmlTextContent] == 'true';
	var isCheckedOut = response.selectSingleNode("repository/node/isCheckedOut")[org.ametys.servercomm.ServerComm.xmlTextContent] == 'true';
	org.ametys.repository.JCRViewTab.updateToolbar(isLocked, isCheckedOut);
	
	if (this._waitMsg != null)
	{
		this._waitMsg.hide();
		delete this._waitMsg;
	}
}

/**
 * Hide or update the referers panel
 * @param {Ext.grid.GridPanel} The referers grid panel
 * @param referers The XML nodes representing referers of the current node
 */
org.ametys.repository.JCRViewTab.updateReferers = function(panel, referers)
{
	if (referers == null || referers.length == 0)
	{
		panel.hide();
		return;
	}	

	var myData = [];
	for (var i=0; i &lt; referers.length; i++)
	{
		var referer = referers[i];

		var path = referer.getAttribute("pathWithGroups");
		var property = referer.getAttribute("name");
	
		myData.push([path, property]);		
	}	
	
	panel.getStore().loadData(myData);
	panel.setTitle("<i18n:text i18n:key="REFERERS_PANEL_TITLE"/> (" + referers.length + " <i18n:text i18n:key="REFERERS_ELEMENT"/>)");
	panel.show();
}

/**
 * Update the status buttons (lock status and checkin status)
 * @param {Boolean} locked true if the node is locked
 * @param {Boolean} checkedOut true if the node is checked out
 */
org.ametys.repository.JCRViewTab.updateToolbar = function(locked, checkedOut)
{	
	// Lock status
	var lockBtn =  Ext.getCmp('btn-status-lock');
	var addNodeBtn =  Ext.getCmp('btn-node-add');
	var removeNodeBtn =  Ext.getCmp('btn-node-remove');
	var addPropBtn =  Ext.getCmp('btn-property-add');
	if (locked)
	{
		lockBtn.setIcon(context.workspaceContext + '/resources/img/locked_32.png');
		var tooltip = org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="LOCKED_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="LOCKED_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/locked_32.png');
		lockBtn.setTooltip(tooltip);
		// REPOSITORY-75 lockBtn.enable();
		addNodeBtn.disable();
		removeNodeBtn.disable();
		addPropBtn.disable();
	}
	else
	{
		lockBtn.setIcon(context.workspaceContext + '/resources/img/unlock_32.png');
		var tooltip = org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="UNLOCKED_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="UNLOCKED_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/unlock_32.png');
		lockBtn.setTooltip(tooltip);
		// REPOSITORY-75 lockBtn.disable();
        addNodeBtn.enable();
        removeNodeBtn.enable();
        addPropBtn.enable();
	}
	
	// Checkout status
	var checkoutBtn =  Ext.getCmp('btn-status-checkout');
	if (checkedOut)
	{
		checkoutBtn.setIcon(context.workspaceContext + '/resources/img/checkout_32.png');
		var tooltip = org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="CHECKED_OUT_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="CHECKED_OUT_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/checkout_32.png');
		checkoutBtn.setTooltip(tooltip);
		checkoutBtn.disable();
	}
	else
	{
		checkoutBtn.setIcon(context.workspaceContext + '/resources/img/checkin_32.png');
		var tooltip = org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="CHECKED_IN_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="CHECKED_IN_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/checkin_32.png');
		checkoutBtn.setTooltip(tooltip);
		checkoutBtn.enable();
	}
}

org.ametys.repository.JCRViewTab.goToNode  = function (workspaceName, uuid)
{
	var serverMessage = new org.ametys.servercomm.ServerMessage('_' + context.workspaceName, '/repository/path-by-uuid',
			{
				workspace: workspaceName,
				uuid: uuid
			}, 
			org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS,  
			null, 
			this, 
			null);
	
	var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
	
	if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key="ERROR_MSG_SOLVE_UUID"/>", response, "org.ametys.repository.actions.SolveUUID"))
	{
		return;
	}
	
	var path = response.selectSingleNode("path")[org.ametys.servercomm.ServerComm.xmlTextContent];
	org.ametys.repository.JCRViewTab._jcrTree.selectNodeByPath (path);
}


org.ametys.repository.JCRViewTab._selectJCRNode = function (sm, node)
{
	if (node != null)
	{
	    var path = node.text;
	    if (node.attributes.pathWithGroups != '')
        {
	        path = node.attributes.pathWithGroups;
        }
		this._jcrTree.selectNodeByPath (path);
	}
}

org.ametys.repository.JCRViewTab._scrollToBottom = function ()
{
	div = this._searchPanel.parent.getEl().dom.childNodes[0].childNodes[0];
	div.scrollTop = div.scrollHeight;
}

org.ametys.repository.JCRViewTab._expandSearch = function ()
{
	this._searchTree.setHeight (200);
}

org.ametys.repository.JCRViewTab._collapseSearch = function ()
{
	this._searchTree.setHeight (50);
}


/************************************/
/**			TOOLBAR ACTIONS			*/
/************************************/

// refresh node
org.ametys.repository.JCRViewTab._refreshNode = function ()
{
	var node = this._jcrTree.getSelectionModel().getSelectedNode();
	node.reload(function (){node.select()});
	org.ametys.repository.JCRViewTab._updateNodeProperties (this, node)
	
}

// add node
org.ametys.repository.JCRViewTab._addNode = function ()
{
	var node = this._jcrTree.getSelectionModel().getSelectedNode();
    if (node.attributes.tagName != 'group')
    {
        org.ametys.repository.actions.AddNode(node, this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._addNodeCb.createDelegate(this));
    }
}

org.ametys.repository.JCRViewTab._addNodeCb = function(path)
{
    org.ametys.repository.JCRViewTab.nodesPendingChanges.push(path);
    org.ametys.repository.JCRViewTab._setPendingChanges(true);
    this._jcrTree.selectNodeByPath(path);
}

// remove node
org.ametys.repository.JCRViewTab._removeNode = function ()
{
	var node = this._jcrTree.getSelectionModel().getSelectedNode();
    if (node.attributes.tagName != 'group')
    {
	    org.ametys.repository.actions.RemoveNode (node, this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._removeNodeCb.createDelegate(this));
    }
}

org.ametys.repository.JCRViewTab._removeNodeCb = function(path, parentTreeNodePath, parentJcrNodePath)
{
    if (org.ametys.repository.JCRViewTab.nodesPendingChanges.indexOf(path) &gt;= 0)
    {
        org.ametys.repository.JCRViewTab.nodesPendingChanges.remove(path);
    }
    else
    {
        org.ametys.repository.JCRViewTab.nodesPendingChanges.push(parentJcrNodePath);
        org.ametys.repository.JCRViewTab._setPendingChanges(true);
        
        var parentNode = this._jcrTree.getNodeById(parentJcrNodePath);
        if (parentNode != null)
        {
            parentNode.getUI().addClass('jcr-tree-node-new');
            parentNode.setIcon(getWorkspaceResources() + '/img/jcr_node_temp_16.png');
        }
    }
    
    if (org.ametys.repository.JCRViewTab.nodesPendingChanges.length &lt; 1 &amp;&amp; !org.ametys.repository.JCRViewTab.hasModifiedAttributes)
    {
        org.ametys.repository.JCRViewTab._setPendingChanges(false);
    }
    this._jcrTree.selectNodeByPath(parentTreeNodePath);
}

// checkout node
org.ametys.repository.JCRViewTab._checkOutNode = function (btn, state)
{
	var node = this._jcrTree.getSelectionModel().getSelectedNode();
    if (node.attributes.tagName != 'group')
    {
        org.ametys.repository.actions.CheckoutNode (node, this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._checkOutNodeCb.createDelegate(this, [btn]));
    }
}
org.ametys.repository.JCRViewTab._checkOutNodeCb = function (btn)
{
	// Update button
	btn.setIcon(context.workspaceContext + '/resources/img/checkout_32.png');
	var tooltip = org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="CHECKED_OUT_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="CHECKED_OUT_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/checkout_32.png');
	btn.setTooltip(tooltip);
	btn.disable();
}

// unlock node
org.ametys.repository.JCRViewTab._unlockNode = function (btn, state)
{
	var node = this._jcrTree.getSelectionModel().getSelectedNode();
    if (node.attributes.tagName != 'group')
    {
        org.ametys.repository.actions.UnlockNode (node, this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._unlockNodeCb.createDelegate(this, [btn]));
    }
}
org.ametys.repository.JCRViewTab._unlockNodeCb = function (btn)
{
	// Update button
	btn.setIcon(context.workspaceContext + '/resources/img/unlock_32.png');
	var tooltip = org.ametys.repository.utils.Tooltip.createTooltipContent ("<i18n:text i18n:key="UNLOCKED_BUTTON_TOOLTIP_TITLE"/>", "<i18n:text i18n:key="UNLOCKED_BUTTON_TOOLTIP_TEXT"/>", context.workspaceContext + '/resources/img/unlock_32.png');
	btn.setTooltip(tooltip);
	btn.disable();
}

// add property
org.ametys.repository.JCRViewTab._addProperty = function ()
{
	var node = this._jcrTree.getSelectionModel().getSelectedNode();
    if (node.attributes.tagName != 'group')
    {
        org.ametys.repository.actions.AddProperty (node, this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._addPropertyCb.createDelegate(this));
    }
}
org.ametys.repository.JCRViewTab._addPropertyCb = function (node)
{
	// reload node properties
	org.ametys.repository.JCRViewTab._updateNodeProperties (this, node);
	
	org.ametys.repository.JCRViewTab._setPendingChanges(true);
}

// Save session.
org.ametys.repository.JCRViewTab._saveSession = function()
{
    org.ametys.repository.actions.SaveSession(this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._saveSessionCb.createDelegate(this));
}
org.ametys.repository.JCRViewTab._saveSessionCb = function()
{
    org.ametys.repository.JCRViewTab.hasModifiedAttributes = false;
    org.ametys.repository.JCRViewTab._setPendingChanges(false);
    
    // Change the icon and remove italic style on all the newly saved nodes.
    for (var i = 0; i &lt; org.ametys.repository.JCRViewTab.nodesPendingChanges.length; i++)
    {
        var path = org.ametys.repository.JCRViewTab.nodesPendingChanges[i];
        var node = this._jcrTree.getNodeById(path);
        if (node != null)
        {
            node.reload();
            node.setIcon(getWorkspaceResources() + '/img/jcr_node_16.png');
            node.getUI().removeClass('jcr-tree-node-new');
        }
    }
    
    org.ametys.repository.JCRViewTab.nodesPendingChanges = [];
    
    // Reload the properties tab.
    org.ametys.repository.JCRViewTab._refreshNode.createDelegate(this).call();
}

// Rollback session.
org.ametys.repository.JCRViewTab._rollbackSession = function()
{
    Ext.Msg.confirm(
        "<i18n:text i18n:key='ROLLBACK_SESSION_CONFIRM_TITLE'/>",
        "<i18n:text i18n:key='ROLLBACK_SESSION_CONFIRM_MSG'/>",
        function(button) {
            if (button == 'yes')
            {
                org.ametys.repository.actions.RollbackSession(this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._rollbackSessionCb.createDelegate(this));
            }
        },
        this
    );
}

org.ametys.repository.JCRViewTab._rollbackSessionCb = function()
{
    org.ametys.repository.JCRViewTab.hasModifiedAttributes = false;
    org.ametys.repository.JCRViewTab._setPendingChanges(false);
    
    // Remove all pending nodes.
    for (var i = 0; i &lt; org.ametys.repository.JCRViewTab.nodesPendingChanges.length; i++)
    {
        var path = org.ametys.repository.JCRViewTab.nodesPendingChanges[i];
        var node = this._jcrTree.getNodeById(path);
        if (node != null)
        {
            node.reload();
            node.setIcon(getWorkspaceResources() + '/img/jcr_node_16.png');
            node.getUI().removeClass('jcr-tree-node-new');
        }
    }
    
    org.ametys.repository.JCRViewTab.nodesPendingChanges = [];
    
    // Reload the properties tab.
    org.ametys.repository.JCRViewTab._refreshNode.createDelegate(this).call();
}

org.ametys.repository.JCRViewTab._onSortChange = function(item, event)
{
    var node = this._jcrTree.getSelectionModel().getSelectedNode();
    
    // Get the first parent which is not a group.
    var jcrNode = node;
    while (jcrNode.attributes.tagName == 'group')
    {
        jcrNode = jcrNode.parentNode;
    }
    
    if (!jcrNode.attributes.order)
    {
        jcrNode.attributes.order = org.ametys.repository.defaultSort;
    }
    
    // If the order was changed, refresh the node and change the tooltip.
    if (jcrNode.attributes.order != item.order)
    {
        jcrNode.attributes.order = item.order;
        
        var menuItem = Ext.getCmp('sort-item');
        
        menuItem.setIcon(item.icon);
        menuItem.setTooltip(org.ametys.repository.utils.Tooltip.createTooltipContent(item.text, item.description, item.icon));
        
        jcrNode.reload(function (){ jcrNode.select() });
    }
}

// execute query
org.ametys.repository.JCRViewTab._executeQuery = function ()
{
	org.ametys.repository.actions.ExecuteQuery (this._searchTree, this._jcrTree._workspaceName);
}

// solve uuid
org.ametys.repository.JCRViewTab._solveUUID = function ()
{
	org.ametys.repository.actions.SolveUUID (this._jcrTree._workspaceName, org.ametys.repository.JCRViewTab._solveUUIDCb.createDelegate(this));
}
org.ametys.repository.JCRViewTab._solveUUIDCb = function (path)
{
	this._jcrTree.selectNodeByPath (path);
}

org.ametys.repository.JCRViewTab._onShowSaveSession = function(item)
{
    var serverMessage = new org.ametys.servercomm.ServerMessage('_' + context.workspaceName,
       'repository/session-information',
        {},
        org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS,
        null,
        this,
        null
    );
    
    var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
    
    if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key='ERROR_MSG_IS_STANDALONE'/>", response, "org.ametys.cms.tool.user.Edit"))
    {
        return;
    }
    
    var hasPendingChanges = response.selectSingleNode('session').getAttribute('hasPendingChanges') == 'true';
    var newNodes = response.selectNodes('session/new-nodes/node');
    
    org.ametys.repository.JCRViewTab.nodesPendingChanges = [];
    
    for (var i = 0; i &lt; newNodes.length; i++)
    {
        org.ametys.repository.JCRViewTab.nodesPendingChanges.push(newNodes[i].getAttribute('path'));
    }
    
    org.ametys.repository.JCRViewTab._setPendingChanges(hasPendingChanges);
}

org.ametys.repository.JCRViewTab.attributeModified = function()
{
    org.ametys.repository.JCRViewTab.hasModifiedAttributes = true;
    org.ametys.repository.JCRViewTab._setPendingChanges(true);
}

org.ametys.repository.JCRViewTab._setPendingChanges = function(hasPendingChanges)
{
    org.ametys.repository.JCRViewTab.hasPendingChanges = hasPendingChanges;
    
    var saveButton = Ext.getCmp('btn-save-session');
    var rollbackButton = Ext.getCmp('btn-rollback-session');
    
    saveButton.setDisabled(!hasPendingChanges);
    rollbackButton.setDisabled(!hasPendingChanges);
}

// 
/**
 * Disconnect and delete current session
 */
org.ametys.repository.JCRViewTab._disconnectRepository = function()
{
	Ext.Msg.confirm("<i18n:text i18n:key="REPOSITORY_CONNECT_TITLE"/>", 
			"<i18n:text i18n:key="REPOSITORY_DISCONNECT_CONFIRM"/>", 
			org.ametys.repository.JCRViewTab._disconnectRepositoryConfirm , 
			this
	);
}
org.ametys.repository.JCRViewTab._disconnectRepositoryConfirm = function(btn)
{
	if (btn == 'no')
		return;
	
	// delete session
	var serverMessage = new org.ametys.servercomm.ServerMessage('_' + context.workspaceName, '/repository/logout',
			{}, 
			org.ametys.servercomm.ServerComm.PRIORITY_SYNCHRONOUS,  
			null, 
			this, 
			null);
	
	var response = org.ametys.servercomm.ServerComm.getInstance().send(serverMessage);
	
	if (org.ametys.servercomm.ServerComm.handleBadResponse("<i18n:text i18n:key="ERROR_MSG_DELETE_SESSION"/>", response, "org.ametys.repository.JCRViewTab"))
	{
		return;
	}
	
	// reload web page
	window.location.reload();


}
function getWorkspaceResources()
{
	return context.workspaceContext + '/resources';
}
